"""
Print the correct command for installing a package on the current system. The main
purpose of this is portability of install scripts across systems.

With no arguments, prints install command (eg. "sudo apt install") so you can
concatenate the package name yourself.

If you pass a filename, it is assumed to be a package spec in YAML format with the
following structure:

    name: "generic-package-name"
    Linux:
        name: "linux-name-of-package"
        arch:
            name: "arch-package-name"

        debian:
            name: "debian-package-name"
            buster:
                name: "buster-package-name"

This is a mechanism for dealing with situations where the same package is listed under
different names in different package managers. The most specific package name will be
preferred; in this case the "arch-package-name" will be preferred over
"generic-package-name" on Arch and "buster-package-name" will be preferred on Debian
buster (on other releases, "debian-package-name" will be preferred).

Usage:
    installcmd (-h|--help)
    installcmd [--log LEVEL]
    installcmd update [--log LEVEL]
    installcmd pkgspec PKG_INFO [--log LEVEL]

Options:
    --log LEVEL  Minimum level of logs to print [default: INFO]
"""
import logging

from docopt import docopt

from installcmd import log
from installcmd.lib import install_command, load_yaml, update_command


def main():
    args = docopt(__doc__)

    loglvl = args["--log"]
    if loglvl:
        log.setLevel(logging.getLevelName(loglvl.upper()))

    if args["update"]:
        print(update_command())
    elif args["pkgspec"]:
        print(install_command(load_yaml(args["PKG_INFO"])))
    else:
        print(install_command())



import platform
from pathlib import Path
from typing import Dict, Union, List

import yaml

from installcmd import log, linux

RELEASE_SPEC = Dict[str, str]
DISTRO_SPEC = Dict[str, Union[str, RELEASE_SPEC]]
OS_SPEC = Dict[str, Union[str, DISTRO_SPEC]]


def install_command(pkg_spec: Dict[str, OS_SPEC] = None) -> str:
    """
    Return correct install command for this system. If package spec is given, will
    also attempt to select appropriate package name from the spec. Otherwise will
    only return the install command.

    If either spec fails to match any commands/names at all, will crash.

    :param pkg_spec: A parsed package spec (optional)
    :return: Correct install command for this system.
    """

    cmd_spec = load_yaml(Path(__file__).parent / "commands.yaml")
    install = apply_spec(cmd_spec, "install")
    if install is None:
        log.error(f"No known install command for this platform")
        exit(1)

    if pkg_spec is None:
        return install

    # If package spec is given, handle that as well
    package = apply_spec(pkg_spec, "name")
    if package is None:
        raise Exception("Could not find valid package name.")

    return f"{install} {package}"


def update_command() -> str:
    """
    Return correct update (refresh package cache) command for this system. If spec
    fails to match any commands at all, will crash.

    :return: Correct update command for this system.
    """

    cmd_spec = load_yaml(Path(__file__).parent / "commands.yaml")
    update = apply_spec(cmd_spec, "update")
    if update is None:
        raise Exception("Could not find valid update command.")

    return update


def apply_spec(spec: Dict[str, OS_SPEC], key: str) -> str:
    """
    Read OS, distribution and release.

    The implementation uses os/distro/release terminology, which is Linux-specific.
    However we are actually calling Python's platfrom uname, node and release which
    should be platform independent (at least in theory). Since Linux OSes have
    vastly more version diversity, there's not really a commonly used Kernel-agnostic
    terminology, so I decided to just pretend the Linux terms apply to all OSes. Again,
    this is a matter of comments and variable names, as the code itself is meant
    to be OS-agnostic.
    """
    # Try looking for successively less qualified commands
    path = [os_name(), distro_name(), platform.uname().release]
    while path:
        str_path = "/".join(path)
        release_cmd = dict_get_path(spec, path + [key])
        if release_cmd:
            log.info(f'Found "{release_cmd}" at {str_path}')
            return release_cmd
        log.debug(f"Couldn't find key: {str_path}")
        path.pop()

    # At this point, you would return an OS-independent generic command, but if
    # such a thing existed this package would not be necessary :)
    log.error(f"Couldn't match this platform. Returning None.")
    return None


def os_name():
    # In Docker containers, uname returns host's OS name
    if linux.is_linux():
        return "linux"
    else:
        return platform.uname().system


def distro_name():
    if linux.is_linux():
        return linux.distro_name()
    else:
        return platform.uname().node


def dict_get_path(d: dict, key_path: List[str]):
    """
    Descend down the path of keys in the dictionary and retrieve value. If not
    found, return None.
    """
    for k in key_path:
        if k not in d:
            return None
        d = d[k]
    return d


def load_yaml(yaml_path: str):
    with open(yaml_path) as f:
        y = yaml.safe_load(f)
    return y



"""
Linux-specifc OS detection methods.
"""
import dotenv
from pathlib import Path
from installcmd import log


def osrel():
    return Path("/etc/os-release")


def lsbrel():
    return Path("/etc/lsb-release")


def is_linux():
    """
    Return true (hopefully) if OS is Linux.

    TODO: Handle corner cases as they arise.
    """
    if osrel().exists() or lsbrel().exists():
        return True

    return False


def distro_name():
    # /etc/os-release is more reliable than uname eg. in Docker containers
    # See also https://unix.stackexchange.com/q/92199
    if osrel():
        osrel_file = dotenv.dotenv_values(osrel())
        return osrel_file["ID"]
    log.warning("Couldn't find distro name.")
    return None



from pathlib import Path

import setuptools

setuptools.setup(
    name="installcmd",
    version="0.2.3",
    author="Azat Akhmetov",
    description="Print correct command for installing a package.",
    long_description=Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    url="https://github.com/metov/installcmd",
    packages=setuptools.find_packages(),
    package_data={"": ["commands.yaml"]},
    entry_points={"console_scripts": ["installcmd = installcmd.cli:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        "coloredlogs",
        "docopt",
        "python-dotenv",
        "pyyaml",
    ],
)
